#include "BranchChart.h"

extern BranchLoadWindow* BranchLoadWindow1;
extern MainWindow* MainWindow1;
extern DetailWindow* DetailWindow1;

BEGIN_EVENT_TABLE(BranchChart, wxGLCanvas)
    EVT_SIZE(BranchChart::OnSize)
    EVT_PAINT(BranchChart::OnPaint)
    EVT_ERASE_BACKGROUND(BranchChart::OnEraseBackground)
	//EVT_LEFT_DCLICK(BranchChart::OnDoubleClick)
	EVT_LEFT_DOWN(BranchChart::OnClick)
	//EVT_RIGHT_DOWN(BranchChart::OnClick)
	//EVT_KEY_DOWN(BranchChart::OnKeyDown)
	//EVT_MOTION(BranchChart::OnMouseMotion)
END_EVENT_TABLE()

extern FTFont *font10, *font18, *font12;

extern int 	timeframe,
			selected_branch;
			
extern BRANCH_DSP branch_dsp[BRANCH_NUM];
extern GLfloat carregamento_overload;

BranchChart::BranchChart(wxWindow *parent, wxGLContext* other, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxGLCanvas(parent, other, id, pos, size, style|wxFULL_REPAINT_ON_RESIZE , name )
{
    m_init = false;

	m_xmin = 0.0;
	m_ymin = 0.0;
	m_xmax = 100.0;
	m_ymax = 100.0;
	
	// copy strings
	char _t[] = "Branch Load (h x MW)";
	char _tx[] = "Hour";
	char _ty[] = "Load";
		
	m_title = new char[strlen(_t)+1]; strcpy(m_title, _t);
	m_xtitle = new char[strlen(_tx)+1]; strcpy(m_xtitle, _tx);
	m_ytitle = new char[strlen(_ty)+1]; strcpy(m_ytitle, _ty);
	
	// get text width & height
	{
		float x1, y1, z1, x2, y2, z2;
	
		font18->BBox(m_title, x1, y1, z1, x2, y2, z2);
		m_titlew = x2 - x1;
		m_titleh = y2 - y1;
		
		font12->BBox(m_xtitle, x1, y1, z1, x2, y2, z2);
		m_xtitlew = x2 - x1;
		m_xtitleh = y2 - y1;
		
		m_ytitlew = font12->FaceSize() * 2/3;
		m_ytitleh = font12->FaceSize() * strlen(m_ytitle);
	}
	
	line = 0;
	horario = -1;
}


BranchChart::~BranchChart()
{
	delete[] m_title;
	delete[] m_xtitle;
	delete[] m_ytitle;
}

void BranchChart::OnPaint( wxPaintEvent& WXUNUSED(event) )
{
	wxPaintDC dc(this);

	if (!GetContext()) return;

	SetCurrent();
    // Init OpenGL once, but after SetCurrent
	if (!m_init)
	{
		InitGL();
		glClearColor (bkg1[0], bkg1[1], bkg1[2], bkg1[3]);
		m_init = true;
		Zoom();
	}

	glClear(GL_COLOR_BUFFER_BIT);
	DrawChart();
	SwapBuffers();
}

void BranchChart::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);
	Zoom();
	//event.Skip();
}

void BranchChart::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
  // Do nothing, to avoid flashing.
}

void BranchChart::InitGL()
{
//    SetCurrent();
}

void BranchChart::OnClick(wxMouseEvent &event)
{
//void processMouse_ev(int button, int state, int x, int y) 
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy; /* OpenGL y coordinate position */
	GLdouble wx, wy, wz; /* returned world x, y, z coords */
	int aux, i;
	
	
	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);
	
	step_ev = branch_dsp[selected_branch].max_pot/timeframe;
	shift_ev = 0.18*branch_dsp[selected_branch].max_pot;

	/* viewport[3] is height of window in pixels */
	realy = viewport[3] - (GLint) event.m_y - 1;
	
	gluUnProject((GLdouble) event.m_x, (GLdouble) realy, 0.0, mvmatrix, projmatrix, viewport, &wx, &wy, &wz);
	
	//Plot the values on the screen
	if (event.LeftDown())
	{
		line = 1;
		horario = int((wx-shift_ev-step_ev)/step_ev);
		if (horario<0) horario=0;
		if (horario>23) horario=23;
		//Plot the mouse line
		DrawMouseLine(wx);
		MainWindow1->Canvas1->Refresh(false);
		DetailWindow1->Refresh(false);
	}
	else 
	{
		line = 0;
	}
	
	BranchLoadWindow1->scHour->SetValue(horario);
	Refresh(false);
}

void BranchChart::DrawChart()
{
	char tmp[30], branch_load_txt[30];
//	char *aux_str;
	
	int evolubranch = selected_branch;
	// --->Starts here former "void Plot_Axes_Evolution(int evolubranch)"
    int i;
    double aux1;

	gcvt(branch_dsp[selected_branch].pot[horario], 7, tmp);
	BranchLoadWindow1->lbBranchLoad->SetLabel(wxString::FromAscii(tmp));
	
    step_ev = branch_dsp[evolubranch].max_pot/timeframe;
    shift_ev = 0.18*branch_dsp[evolubranch].max_pot;
    
    glColor3fv(axes_color);
    
    //Plot Vertical lines
	glColor3fv(axes_color);
	
	for (i=0; i<26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(i*step_ev + shift_ev, shift_ev);
		glVertex2f(i*step_ev + shift_ev, 25*step_ev + shift_ev); 
		glEnd();
	}
	for (i=0; i<26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(shift_ev, i*step_ev + shift_ev);
		glVertex2f(25*step_ev + shift_ev, i*step_ev + shift_ev); 
		glEnd();
	}  	
	
    /*for (i = 0; i < 26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(i*step_ev + shift_ev, shift_ev+5*step_ev);
		glVertex2f(i*step_ev + shift_ev, 24*step_ev + shift_ev); 
		glEnd();
    }
    //Plot Horizontal lines
    for (i = 5; i < 25; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(shift_ev, i*step_ev + shift_ev);
		glVertex2f(25*step_ev + shift_ev, i*step_ev + shift_ev); 
		glEnd();
    }  
	*/
    if (line) DrawMouseLine(shift_ev + (horario+1)*step_ev);
    
    glColor3fv(title_color);
	DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_titlew*m_bw)/2, m_ymax - (shift_ev + m_titleh*m_bh)/2, m_title, font18);
    
    //Plot text lables
    glColor3fv(title_color);
	DRAWTEXTV(m_xmin + (shift_ev - m_ytitlew*m_bw)/2, m_ymin + (m_ymax - m_ymin - shift_ev + m_ytitleh*m_bh)/2, m_ytitle, font12, m_bh);
	DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_xtitlew*m_bw)/2, m_ymin + (shift_ev - m_xtitleh*m_bh)/2, m_xtitle, font12);

	// --->Starts here former "void Plot_Graph_Evolution(int evolubranch)"
    glColor3fv(graphic_color);
    
	glBegin(GL_LINE_STRIP);
    for (i=0; i<timeframe; i++)
	{
		glVertex2f(shift_ev + step_ev*(i+1), shift_ev + branch_dsp[evolubranch].pot[i]);
    }
    glEnd();
    
    for (i = 0; i < timeframe; i++)
	{
		glColor3fv(bkg2);
		glPointSize(3.0);
		if ((branch_dsp[evolubranch].pot[i]) >= ((carregamento_overload/100)*branch_dsp[evolubranch].max_pot)) 
		{
			glColor3f(1.0,0.0,0.0);
			glPointSize(5.0);
		}
		glBegin(GL_POINTS);
		glVertex2f(shift_ev + step_ev*(i+1), shift_ev  + branch_dsp[evolubranch].pot[i]);
		glEnd();
		glPointSize(1.0);
    }

}

void BranchChart::DrawMouseLine(GLfloat x)
{
	int i;
	
	step_ev = branch_dsp[selected_branch].max_pot/timeframe;
	shift_ev = 0.18*branch_dsp[selected_branch].max_pot;
	
	if ((x >= shift_ev) && (x < shift_ev + 25*step_ev)) 
	{
		glColor4f(1.0, 0.0, 0.0, 0.1);
		glBegin(GL_LINES);
		glVertex2f(x, m_ymax - shift_ev);
		glVertex2f(x, m_ymin + shift_ev);
		glEnd();
	}
	
}



void BranchChart::Zoom(int w, int h)
{
	int w_, h_;
	
	w_ = w;
	h_ = h;
	if (w==-1 || h==-1)
	{
		GetClientSize(&w_, &h_);
	}
	
    if (!GetContext()) return;
	SetCurrent();
	glViewport(0, 0, w_, h_);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	m_xmax = 1.4*branch_dsp[selected_branch].max_pot;
	m_ymax = 1.4*branch_dsp[selected_branch].max_pot;
			
	m_bh = (m_ymax - m_ymin)/h_;
	m_bw = (m_xmax - m_xmin)/w_;
	
	gluOrtho2D(m_xmin, m_xmax, m_ymin, m_ymax);
	glMatrixMode(GL_MODELVIEW);		
}
