#include "DemandChart.h"

extern MainWindow* MainWindow1;
extern DetailWindow* DetailWindow1;

extern FTFont *font10, *font18, *font12;
extern int 	horario, timeframe, info, line, 
				show_interval;
extern GLfloat demanda_L[TIMEFRAME], max_demanda_L;

const char time_format[] = "Time: %d h";
const char demand_format[] ="Power Demand: %'.2f MW";

BEGIN_EVENT_TABLE(DemandChart, wxGLCanvas)
    EVT_SIZE(DemandChart::OnSize)
    EVT_PAINT(DemandChart::OnPaint)
    EVT_ERASE_BACKGROUND(DemandChart::OnEraseBackground)
    //EVT_ENTER_WINDOW(DemandChart::OnEnterWindow)
	//EVT_LEFT_DCLICK(DemandChart::OnDoubleClick)
	EVT_LEFT_DOWN(DemandChart::OnClick)
	//EVT_RIGHT_DOWN(DemandChart::OnClick)
	EVT_KEY_DOWN(DemandChart::OnKeyDown)
	EVT_MOTION(DemandChart::OnMouseMotion)
END_EVENT_TABLE()

DemandChart::DemandChart(wxWindow *parent, wxGLContext* other, wxWindowID id,const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxGLCanvas(parent, other, id, pos, size, style|wxFULL_REPAINT_ON_RESIZE , name )
{
	m_init = false;

	info1 = 0;
	m_xmin = 0.0;
	m_xmax = 100.0;
	m_ymin = 0.0;
	m_ymax = 100.0;
	
	// copy strings
	char _t[] = "Power Demand (h x MW)";
	char _tx[] = "Hour";
	char _ty[] = "Power Demand";
		
	m_title = new char[strlen(_t)+1]; strcpy(m_title, _t);
	m_xtitle = new char[strlen(_tx)+1]; strcpy(m_xtitle, _tx);
	m_ytitle = new char[strlen(_ty)+1]; strcpy(m_ytitle, _ty);
	
	m_time = new char[strlen(time_format)+1];
	m_demand = new char[strlen(demand_format)+6];
	// get text width & height
	{
		float x1, y1, z1, x2, y2, z2;
	
		font18->BBox(m_title, x1, y1, z1, x2, y2, z2);
		m_titlew = x2 - x1;
		m_titleh = y2 - y1;
		
		font12->BBox(m_xtitle, x1, y1, z1, x2, y2, z2);
		m_xtitlew = x2 - x1;
		m_xtitleh = y2 - y1;
		
		m_ytitlew = font12->FaceSize() * 2/3;
		m_ytitleh = font12->FaceSize() * strlen(m_ytitle);
	}
	
}


DemandChart::~DemandChart()
{
	delete[] m_title;
	delete[] m_xtitle;
	delete[] m_ytitle;
	delete[] m_time;
	delete[] m_demand;
}


void DemandChart::OnPaint( wxPaintEvent& WXUNUSED(event) )
{
	wxPaintDC dc(this);

	if (!GetContext()) return;

	SetCurrent();
    // Init OpenGL once, but after SetCurrent
	if (!m_init)
	{
		InitGL();

		glClearColor (bkg1[0], bkg1[1], bkg1[2], bkg1[3]);
	
		m_init = true;
		
		Zoom();		
	}

	glClear(GL_COLOR_BUFFER_BIT);
	DrawChart();
	SwapBuffers();
}

void DemandChart::OnSize(wxSizeEvent& event)
{
	// this is also necessary to update the context on some platforms
	wxGLCanvas::OnSize(event);
	Zoom();
	//event.Skip();
}

void DemandChart::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
  // Do nothing, to avoid flashing.
}

void DemandChart::InitGL()
{
//    SetCurrent();
}

void DemandChart::OnClick(wxMouseEvent &event)
{
//void processMouse1(int button, int state, int x, int y) 
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy; /* OpenGL y coordinate position */
	GLdouble wx, wy, wz; /* returned world x, y, z coords */
    
	if (!GetContext()) return;
    SetCurrent();


	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);
	
	/* viewport[3] is height of window in pixels */
	realy = viewport[3] - (GLint) event.m_y - 1;
	
	gluUnProject((GLdouble) event.m_x, (GLdouble) realy, 0.0, mvmatrix, 
		projmatrix, viewport, &wx, &wy, &wz);
	
	//Calculates the demand and puts it on a string
	sprintf(m_demand, demand_format, float(wy - shift1));
	
	//Calculates the hour and puts it on a string
	sprintf(m_time, time_format, int((wx - shift1 - step1)/step1));
	
	//Plot the values on the screen
	if (show_interval) 
	{
		if (event.LeftDown())
		{
			line = 1;
			horario = int((wx-shift1-step1)/step1);
			if (horario<0) horario=0;
			if (horario>23) horario=23;
		}
		else
		{
			line = 0;
		}
		MainWindow1->scHour->SetValue(horario);
		
		Zoom();
		MainWindow1->Refresh(false);
	}
}



void DemandChart::Zoom(int w, int h)
{
	int w_, h_;
	
	w_ = w;
	h_ = h;
	if (w==-1 || h==-1)
	{
		GetClientSize(&w_, &h_);
	}
	
    step1 = max_demanda_L/timeframe;
    shift1 = 0.15*max_demanda_L;

    if (GetContext())
    {
        SetCurrent();
		glViewport(0, 0, w_, h_);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		m_xmax = 1.4*max_demanda_L;
		m_ymax = 1.4*max_demanda_L;
		m_bh = (m_ymax - m_ymin)/h_;
		m_bw = (m_xmax - m_xmin)/w_;
				
		gluOrtho2D(m_xmin, m_xmax, m_ymin, m_ymax);
		glMatrixMode(GL_MODELVIEW);
    }	
}

void DemandChart::OnKeyDown(wxKeyEvent& event) //void keyboard1(unsigned char key, int x, int y)
{
	switch (event.GetKeyCode()) {
	case 'l':
	case 'L':
		line = (line) ? 0 : 1;
		break;
	case WXK_RIGHT:
		if (show_interval)
		{
			line = 1;
			horario++;
			if (horario>23) horario = 0;
		}
		break;
	case WXK_LEFT:
		if (show_interval) 
		{
			line = 1;
			horario--;
			if (horario<0) 
			{
				horario = 23;
			}
		}
		break;
	case WXK_HOME:
		line = 0;
		break;
	}
	MainWindow1->Refresh(false);
	DetailWindow1->Refresh(false);
}


void DemandChart::DrawChart()
{
	unsigned int i;
	
	// --->Starts here former "void Plot_Axes_L(void)"
	glColor3fv(axes_color);
	
	for (i=0; i<26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(i*step1 + shift1, shift1);
		glVertex2f(i*step1 + shift1, 25*step1 + shift1); 
		glEnd();
	}
	for (i=0; i<26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(shift1,            i*step1 + shift1);
		glVertex2f(25*step1 + shift1, i*step1 + shift1); 
		glEnd();
	}  
	
	sprintf(m_demand, demand_format, demanda_L[horario]);
	
	//Plot the red line and the information
	if (line == 1)
	{
		info1 = 1;
		DrawMouseLine(shift1 + (horario+1)*step1, demanda_L[horario]);
	}
	//or the title only
	else 
	{
		glColor3fv(title_color);
		DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_titlew*m_bw)/2, m_ymax - (shift1 + m_titleh*m_bh)/2, m_title, font18);
	}
	
	//Plot text lables
	glColor3fv(title_color);
	DRAWTEXTV(m_xmin + (shift1 - m_ytitlew*m_bw)/2, m_ymin + (m_ymax - m_ymin - shift1 + m_ytitleh*m_bh)/2, m_ytitle, font12, m_bh);
	DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_xtitlew*m_bw)/2, m_ymin + (shift1 - m_xtitleh*m_bh)/2, m_xtitle, font12);
		
	// Names from:
	// http://www.rte-france.com/jsp/an/courbes/courbes.jsp
	
	// --->Starts here former "void Plot_Graphic_L(void)"
    glColor3fv(graphic_color);

    glBegin(GL_LINE_STRIP);
    for (i=0; i < timeframe; i++)
	{
		glVertex2f(shift1 + step1*(i+1), shift1 + demanda_L[i]);
    }
    glEnd();

    glColor3fv(bkg2);
    glPointSize(2.0);
    glBegin(GL_POINTS);
    for (i = 0; i < timeframe; i++)
	{
		glVertex2f(shift1 + step1*(i+1), shift1 + demanda_L[i]);
    }
    glEnd();
    glPointSize(1.0);	

}

void DemandChart::DrawMouseLine(GLfloat x, GLfloat y)
{
	sprintf(m_time, time_format, horario);
	
	if ((x >= shift1)&&(x < shift1 + 25*step1)) 
	{
		glColor4f(1.0, 0.0, 0.0, 0.1);
		glBegin(GL_LINES);
		glVertex2f(x, step1+shift1+max_demanda_L);
		glVertex2f(x, shift1);
		glEnd();
		
		if (info1) {
			glColor3fv(title_color);
			DRAWTEXT(m_xmin + shift1, m_ymax - shift1, m_demand, font10);
//			DRAWTEXT(m_xmin + shift1, m_ymax - shift1, m_time, font10);
			DRAWTEXT(m_xmin + shift1, m_ymin + shift1 - font10->FaceSize()*m_bh, m_time, font10);
		}
	}	
}


void DemandChart::OnMouseMotion(wxMouseEvent &event)//void processMotion1(int x, int y) 
{
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy; /* OpenGL y coordinate position */
	GLdouble wx, wy, wz; /* returned world x, y, z coords */
	
	if (!show_interval) return;
    if (!GetContext()) return;
    SetCurrent();
	
	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);
	
	/* viewport[3] is height of window in pixels */
	realy = viewport[3] - (GLint) event.m_y - 1;
	
	gluUnProject((GLdouble) event.m_x, (GLdouble) realy, 0.0, mvmatrix, 
		projmatrix, viewport, &wx, &wy, &wz);
	
	//Calculates the demand and puts it on a string
	sprintf(m_demand, demand_format, float(wy - shift1));
	
	//Calculates the hour and puts it on a string
	sprintf(m_time, time_format, int((wx - shift1 - step1)/step1));
	
	//Plot the mouse line
	glClear(GL_COLOR_BUFFER_BIT);
	DrawChart();
	
	info1 = 0;	
	DrawMouseLine(wx, wy);
	SwapBuffers();
	
	if (event.GetEventObject() == this) 
	{
		((LoadChart*)(MainWindow1->Chart2))->OnMouseMotion(event);
	}	
//	MainWindow1->Refresh(false);	
}
