#include "LoadChart.h"

extern MainWindow* MainWindow1;
extern DetailWindow* DetailWindow1;

BEGIN_EVENT_TABLE(LoadChart, wxGLCanvas)
    EVT_SIZE(LoadChart::OnSize)
    EVT_PAINT(LoadChart::OnPaint)
    EVT_ERASE_BACKGROUND(LoadChart::OnEraseBackground)
    //EVT_ENTER_WINDOW(LoadChart::OnEnterWindow)
	//EVT_LEFT_DCLICK(LoadChart::OnDoubleClick)
	EVT_LEFT_DOWN(LoadChart::OnClick)
	//EVT_RIGHT_DOWN(LoadChart::OnClick)
	EVT_KEY_DOWN(LoadChart::OnKeyDown)
	EVT_MOTION(LoadChart::OnMouseMotion)
END_EVENT_TABLE()

const char time_format[] = "Time: %d h";
const char avload_format[]="Average Load: %'.2f%%";
const char maxload_format[]="Maximum Load: %'.2f%%";

extern FTFont *font10, *font18, *font12;

extern int 	horario, 
			timeframe, 
			line, 
			show_interval;

extern GLfloat carregamento_perc[TIMEFRAME];
extern GLfloat max_carregamento_perc[TIMEFRAME];

LoadChart::LoadChart(wxWindow *parent, wxGLContext* other, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name)
	: wxGLCanvas(parent, other, id, pos, size, style|wxFULL_REPAINT_ON_RESIZE, name )
{
    m_init = false;

	info2 = 0;
	step2 = 100.0/timeframe;
    shift2 = 0.15*100.0;

	m_xmin = 0.0;
	m_xmax = 1.4*100.0;
	m_ymin = 0.0;
	m_ymax = 1.4*100.0;

	// create strings
	char _t[] = "Load (h x MW)";
	char _tx[] = "Hour";
	char _ty[] = "Load";
		
	m_title = new char[strlen(_t)+1]; strcpy(m_title, _t);
	m_xtitle = new char[strlen(_tx)+1]; strcpy(m_xtitle, _tx);
	m_ytitle = new char[strlen(_ty)+1]; strcpy(m_ytitle, _ty);
	m_time = new char[strlen(time_format)+1];
	m_load = new char[strlen(avload_format)+6];
	m_maxload = new char[strlen(maxload_format)+6];
	// get text width & height
	{
		float x1, y1, z1, x2, y2, z2;
	
		font18->BBox(m_title, x1, y1, z1, x2, y2, z2);
		m_titlew = x2 - x1;
		m_titleh = y2 - y1;
		
		font12->BBox(m_xtitle, x1, y1, z1, x2, y2, z2);
		m_xtitlew = x2 - x1;
		m_xtitleh = y2 - y1;
		
		m_ytitlew = font12->FaceSize() * 2/3;
		m_ytitleh = font12->FaceSize() * strlen(m_ytitle);
	}	

}

LoadChart::~LoadChart()
{
	delete[] m_title;
	delete[] m_xtitle;
	delete[] m_ytitle;
	delete[] m_time;
	delete[] m_load;
	delete[] m_maxload;
}

void LoadChart::OnPaint(wxPaintEvent& WXUNUSED(event))
{
	wxPaintDC dc(this);

	if (!GetContext()) return;

	SetCurrent();
    // Init OpenGL once, but after SetCurrent
	if (!m_init)
	{
		InitGL();

		glClearColor (bkg1[0], bkg1[1], bkg1[2], bkg1[3]);
	
		m_init = true;
		
		Zoom();		
	}

	glClear(GL_COLOR_BUFFER_BIT);
	DrawChart();
	SwapBuffers();
}

void LoadChart::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);

	Zoom(event.GetSize().GetWidth(), event.GetSize().GetHeight());
//	Refresh(false);
	Update();
}

void LoadChart::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
  // Do nothing, to avoid flashing.
}

void LoadChart::InitGL()
{
//    SetCurrent();
/*	float light1_ambient[4]  = { 1.0, 1.0, 1.0, 1.0 };
	glLightfv(GL_LIGHT1, GL_AMBIENT,  light1_ambient);

	float front_emission[4] = { 0.3, 0.2, 0.1, 0.0 };
	float front_ambient[4]  = { 0.2, 0.2, 0.2, 0.0 };
	float front_diffuse[4]  = { 0.95, 0.95, 0.8, 0.0 };
	float front_specular[4] = { 0.6, 0.6, 0.6, 0.0 };
	glMaterialfv(GL_FRONT, GL_EMISSION, front_emission);
	glMaterialfv(GL_FRONT, GL_AMBIENT, front_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, front_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, front_specular);
	glMaterialf(GL_FRONT, GL_SHININESS, 16.0);
	glColor4fv(front_diffuse);

	glEnable(GL_COLOR_MATERIAL);

	glEnable(GL_LIGHTING);	*/
}

void LoadChart::OnDoubleClick(wxMouseEvent &event)
{

}

void LoadChart::OnClick(wxMouseEvent &event)
{
//void processMouse1(int button, int state, int x, int y) 
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy; /* OpenGL y coordinate position */
	GLdouble wx, wy, wz; /* returned world x, y, z coords */
	int aux;
//	char carregamento_s[30];
	
    if (!GetContext()) return;
    SetCurrent();

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);
	
	/* viewport[3] is height of window in pixels */
	realy = viewport[3] - (GLint) event.m_y - 1;
	
	gluUnProject((GLdouble) event.m_x, (GLdouble) realy, 0.0, mvmatrix, 
				  projmatrix, viewport, &wx, &wy, &wz);

    //Calculates the demand and puts it on a string
/*    aux = int(wy - shift2);
    gcvt(aux, 7, carregamento_s);
    strcat(carregamento_s, " MW");
	*/
    //Calculates the hour and puts it on a string
/*    aux = int((wx - shift2 - step2)/step2);
	sprintf(m_time, time_format, aux);*/
	
	//Plot the values on the screen
	if (show_interval) 
	{
		if (event.LeftDown())
		{
			line = 1;
			horario = int((wx - shift2 - step2) / step2);
			if (horario < 0) horario = 0;
			if (horario > 23) horario= 23;
		}
		else
		{
			line = 0;
		}
		MainWindow1->scHour->SetValue(horario);
		
		Zoom();
		MainWindow1->Refresh(false);
	}
}

void LoadChart::Zoom(int w, int h)
{
    int w_, h_;
	
	w_ = w;
	h_ = h;
	if (w==-1 || h==-1)
	{
    	GetClientSize(&w_, &h_);
	}
	
    if (GetContext())
    {
        SetCurrent();
		glViewport(0, 0, w_, h_);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		
		m_bh = (m_ymax - m_ymin)/h_;
		m_bw = (m_xmax - m_xmin)/w_;
				
		gluOrtho2D(m_xmin, m_xmax, m_ymin, m_ymax);
		glMatrixMode(GL_MODELVIEW);		
    }	
}

void LoadChart::OnKeyDown(wxKeyEvent& event) //void keyboard1(unsigned char key, int x, int y)
{
	switch (event.GetKeyCode()) {
	case 'l':
	case 'L':
		line = (line) ? 0 : 1;
		break;
	case WXK_RIGHT:
		if (show_interval)
		{
			line = 1;
			horario++;
			if (horario>23) horario = 0;
			MainWindow1->scHour->SetValue(horario);
		}
		break;
	case WXK_LEFT:
		if (show_interval) 
		{
			line = 1;
			horario--;
			if (horario<0) 
			{
				horario = 23;
			}
			MainWindow1->scHour->SetValue(horario);
		}
		break;
	case WXK_HOME:
		line = 0;
		break;
	}
	MainWindow1->Refresh(false);
	DetailWindow1->Refresh(false);
	
}


void LoadChart::DrawChart()
{
    int i;
    double aux;

	// --->Starts here former "void Plot_Axes_Carregamento(void)"
    glColor3fv(axes_color);
    
    for (i = 0; i < 26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(i*step2 + shift2, shift2);
		glVertex2f(i*step2 + shift2, 25*step2 + shift2); 
		glEnd();
    }
    for (i = 0; i < 26; i++)
	{
		glBegin(GL_LINES);
		glVertex2f(shift2, i*step2 + shift2);
		glVertex2f(25*step2 + shift2, i*step2 + shift2); 
		glEnd();
    }
	
	sprintf(m_load, avload_format, (float)(carregamento_perc[horario]));
	sprintf(m_maxload, maxload_format, (float)(max_carregamento_perc[horario]));
	
    //Plot the red line and the information
    if (line == 1)
	{
		info2 = 1;
		DrawMouseLine(shift2 + (horario+1)*step2, carregamento_perc[horario]);
    }
    //or the title only
    else 
	{
		glColor3fv(title_color);
		DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_titlew*m_bw)/2, m_ymax - (shift2 + m_titleh*m_bh)/2, m_title, font18);
    }
	
    //Plot text labels
    glColor3fv(title_color);
	DRAWTEXTV(m_xmin + (shift2 - m_ytitlew*m_bw)/2, m_ymin + (m_ymax - m_ymin - shift2 + m_ytitleh*m_bh)/2, m_ytitle, font12, m_bh);
	DRAWTEXT(m_xmin + (m_xmax - m_xmin - m_xtitlew*m_bw)/2, m_ymin + (shift2 - m_xtitleh*m_bh)/2, m_xtitle, font12);

	// --->Starts here former "void Plot_Graphic_Carregamento(void)"
    glColor3fv(graphic_color);
	
    glBegin(GL_LINE_STRIP);
    for (i = 0; i < timeframe; i++)
	{
		glVertex2f(shift2 + step2*(i+1), shift2 + carregamento_perc[i]);
    }
    glEnd();
    
    glColor3f(1.0, 0.0, 0.0);
    glLineWidth(2.0);
    glBegin(GL_LINE_STRIP);
    for (i=0; i<timeframe; i++)
	{
		glVertex2f(shift2 + step2*(i+1), shift2 + max_carregamento_perc[i]);
    }
    glEnd();
    glLineWidth(1.0);
    
    glColor3fv(bkg2);
    glPointSize(2.0);
    glBegin(GL_POINTS);
    for (i = 0; i < timeframe; i++)
	{
		glVertex2f(shift2 + step2*(i+1), shift2 + carregamento_perc[i]);
    }
    glEnd();
    glPointSize(1.0);
    
    glColor3fv(bkg2);
    glPointSize(2.0);
    glBegin(GL_POINTS);
    for (i = 0; i < timeframe; i++)
	{
		glVertex2f(shift2 + step2*(i+1), shift2 + max_carregamento_perc[i]);
    }
    glEnd();
    glPointSize(1.0);	
}

void LoadChart::DrawMouseLine(GLfloat x, GLfloat y)
{
	int i, aux;
	
	sprintf(m_time, time_format, horario);
	if ((x >= shift2) && (x < shift2 + 25*step2)) 
	{
		glColor4f(1.0, 0.0, 0.0, 0.1);
		glBegin(GL_LINES);
		glVertex2f(x, step2 + shift2 + 100.0);
		glVertex2f(x, shift2);
		glEnd();
		
		if (info2) 
		{
			glColor3fv(title_color);			
			
			DRAWTEXT(m_xmin + shift2, m_ymax - shift2 + font10->FaceSize()*m_bh, m_load, font10);
			DRAWTEXT(m_xmin + shift2, m_ymax - shift2, m_maxload, font10);
			DRAWTEXT(m_xmin + shift2, m_ymin + shift2 - font10->FaceSize()*m_bh, m_time, font10);
			
		}
	}
}


void LoadChart::OnMouseMotion(wxMouseEvent &event)//void processMotion1(int x, int y) 
{
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy; /* OpenGL y coordinate position */
	GLdouble wx, wy, wz; /* returned world x, y, z coords */
	
	if (!show_interval) return;
    if (!GetContext()) return;
    SetCurrent();
	
	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);
	
	/* viewport[3] is height of window in pixels */
	realy = viewport[3] - (GLint) event.m_y - 1;
	
	gluUnProject((GLdouble) event.m_x, (GLdouble) realy, 0.0, mvmatrix, projmatrix, 
		viewport, &wx, &wy, &wz);
	
	//Calculates the demand and puts it on a string
	sprintf(m_load, avload_format, (float)(wy - shift2));
	sprintf(m_maxload, maxload_format, (float)(max_carregamento_perc[horario]));
	
	//Calculates the hour and puts it on a string
	sprintf(m_time, time_format, int((wx - shift2 - step2) / step2));	

	//Plot the mouse line
	glClear(GL_COLOR_BUFFER_BIT);
	DrawChart();
	
	info2 = 0;
	DrawMouseLine(wx, wy);
	SwapBuffers();
	
	if (event.GetEventObject() == this) 
	{
		((DemandChart*)(MainWindow1->Chart1))->OnMouseMotion(event);
	}
	//MainWindow1->Refresh(false);	
}
