#include <wx/wx.h>
#include <wx/image.h>

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <wx/splitter.h>
#include <wx/spinctrl.h>

#include "DetailWindow.h"
#include "SimulationWindow.h"
#include "BranchLoadWindow.h"
#include "MainView.h"
#include "LoadChart.h"
#include "DemandChart.h"

class DemandChart;
class LoadChart;
class MainCanvas;

class MainWindow: public wxFrame {
public:
    enum {
        PLLEFTTOP = 1000,
        PLLEFTBOTTOM = 1001,
        PLRIGHT = 1002,
        CKINTERVALMODE = 1003,
        LBHOUR = 1005,
        SCHOUR = 1006,
        BTCHANGECONSTRAINTS = 1008,
        BTTOGGLEANIMATION = 1010,
        LBVOLTAGELEVEL = 1012,
        CEVOLTAGELEVEL = 1013,
		CKSHOWMAP = 1015,
		ANIMATIONTIMER,
		CANVAS1,
		CHART1,
		CHART2
    };
    
	MainWindow(wxWindow* parent, int id, const wxString& title, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize, long style=wxDEFAULT_FRAME_STYLE);
	~MainWindow();
private:
    void set_properties();
    void do_layout();

//protected:
public:
	MainCanvas* Canvas1;
	DemandChart* Chart1;
	LoadChart* Chart2;

    
    wxCheckBox* ckShowMap;
    wxPanel* plLeftTop;
    wxPanel* plLeftBottom;
    wxSplitterWindow* window_2;
    wxPanel* window_1_pane_1;
    wxPanel* plRight;
    wxSplitterWindow* window_1;
    wxCheckBox* ckIntervalMode;
    wxStaticText* lbHour;
    wxSpinCtrl* scHour;
    wxButton* btChangeConstraints;
    wxButton* btToggleAnimation;
    wxStaticText* lbVoltageLevel;
    wxChoice* ceVoltageLevel;
    wxButton* btQuit;

    DECLARE_EVENT_TABLE();

public:
	void OnAnimationTimer(wxTimerEvent& event);
	void OnShowMapCheckBox(wxCommandEvent &event);
	void OnIntervalModeCheckbox(wxCommandEvent &event);
	void OnQuitButton(wxCommandEvent &event);
	void OnChangeConstraintsButton(wxCommandEvent &event);
	void OnVoltageLevelChoice(wxCommandEvent &event);
	void OnHourSpin(wxSpinEvent &event);
	void OnToggleAnimationButton(wxCommandEvent &event);
private:
		wxTimer AnimationTimer;	
		int 	animation;
}; 


#endif // MAINWINDOW_H
