#include "SimulationWindow.h"

extern DetailWindow* DetailWindow1;
extern SimulationWindow* SimulationWindow1;
extern BranchLoadWindow* BranchLoadWindow1;
extern MainWindow* MainWindow1;

extern float sim_gen, old_gen;
extern int sim_bus, horario;
extern BUS_DSP bus_dsp[BUS_NUM];	


SimulationWindow::SimulationWindow(wxWindow* parent, int id, const wxString& title, const wxPoint& pos, const wxSize& size, long style):
    wxFrame(parent, id, title, pos, size, wxDEFAULT_FRAME_STYLE)
{
	_editing = false;
	
    lbBus = new wxStaticText(this, -1, wxT("Bus"));
    scBus = new wxSpinCtrl(this, SCBUS, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
    lbBusGeneration = new wxStaticText(this, -1, wxT("Bus generation"));
    GenerationText = new wxTextCtrl(this, -1, wxT("1.000"));
    GenerationSlider = new wxSlider(this, -1, 1000, -500000, 1000000, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL|wxSL_AUTOTICKS);
    btSimulate = new wxButton(this, BTSIMULATE, wxT("&Simulate"));
    btReset = new wxButton(this, BTRESETSIM, wxT("&Close"));

    set_properties();
    do_layout();
    
	
	extern GLint bus_num;
	scBus->SetRange(1, bus_num);
	btReset->SetLabel(_T("Reset All Data"));
	
}


BEGIN_EVENT_TABLE(SimulationWindow, wxFrame)
    EVT_SPINCTRL(SCBUS, SimulationWindow::OnscBus)
    EVT_TEXT(-1, SimulationWindow::OnGenerationText)
    EVT_SCROLL(SimulationWindow::OnGenerationScroll)
    EVT_BUTTON(BTSIMULATE, SimulationWindow::OnbtSimulate)
    EVT_BUTTON(BTRESETSIM, SimulationWindow::OnbtReset)
	EVT_CLOSE(SimulationWindow::OnClose)	
END_EVENT_TABLE();


void SimulationWindow::set_properties()
{
    // begin wxGlade: SimulationWindow::set_properties
    SetTitle(wxT("VisciPower - Simulation Window"));
    SetSize(wxSize(260, 170));
	GenerationText->SetMinSize(wxSize(50, 22));
	GenerationSlider->SetMinSize(wxSize(120,0));
    // end wxGlade
}


void SimulationWindow::do_layout()
{
    // begin wxGlade: SimulationWindow::do_layout
    wxGridSizer* SizerMain = new wxGridSizer(2, 2, 4, 4);
    wxBoxSizer* sizer_2_copy = new wxBoxSizer(wxVERTICAL);
    SizerMain->Add(lbBus, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SizerMain->Add(scBus, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SizerMain->Add(lbBusGeneration, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	sizer_2_copy->Add(GenerationText, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	sizer_2_copy->Add(GenerationSlider, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SizerMain->Add(sizer_2_copy, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SizerMain->Add(btSimulate, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SizerMain->Add(btReset, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
    SetAutoLayout(true);
    SetSizer(SizerMain);
    Layout();
    Centre();
    // end wxGlade
}

void SimulationWindow::UpdateGeneration(float gen)
{
	_editing = true;
	GenerationSlider->SetValue(int(gen * 1000));
	GenerationText->Clear();
	*GenerationText << gen;
	sim_gen = gen;
	_editing = false;
}

void SimulationWindow::OnscBus(wxSpinEvent &event)
{
	extern int sim_bus;
	
	bus_dsp[sim_bus-1].pot_G[horario] = sim_gen;
	
	sim_bus = event.GetPosition();
	sim_gen = bus_dsp[sim_bus-1].pot_G[horario];
	old_gen = sim_gen;
	
	UpdateGeneration(sim_gen);
}
/*
void SimulationWindow::OnscBusGeneration(wxSpinEvent &event)
{
	sim_gen = scBusGeneration->GetValue();
	old_gen = sim_gen;
	bus_dsp[sim_bus-1].pot_G[horario] = sim_gen;
	
}
*/

void SimulationWindow::OnbtSimulate(wxCommandEvent &event)
{
	vpSimulator();
	vpUpdateVectors();
	
	MainWindow1->Chart1->Zoom();
	MainWindow1->Chart2->Zoom();
//	BranchLoadWindow1->Chart3->Zoom();
	MainWindow1->Refresh(false);
	DetailWindow1->Refresh(false);
}


void SimulationWindow::OnbtReset(wxCommandEvent &event)
{
	vpInitData();
	sim_gen = bus_dsp[sim_bus-1].pot_G[horario];
	old_gen = sim_gen;
	UpdateGeneration(sim_gen);
}


void SimulationWindow::OnClose(wxCloseEvent &event)
{
	event.Veto();
	Hide();
}


void SimulationWindow::OnGenerationText(wxCommandEvent &event)
{
	double d;
	if (!_editing)
	{
		_editing = true;
		if (GenerationText->GetValue().ToDouble(&d))
		{
			GenerationSlider->SetValue(int(d * 1000));
			sim_gen = d;
		}
		else
		{
			GenerationText->Clear();
			*GenerationText << sim_gen;
		}
		_editing = false;
	}
}
void SimulationWindow::OnGenerationScroll(wxScrollEvent &event)
{
	if (!_editing)
	{
		_editing = true;
		GenerationText->Clear();
		sim_gen = float(GenerationSlider->GetValue())/1000.0;
		*GenerationText << sim_gen;
		_editing = false;
	}
}
