#ifndef VPDEFS_H
#define VPDEFS_H

#include <GL/gl.h>
#include <math.h>

#define BUS_NUM 500
#define BRANCH_NUM 1000

/////////////////////!!!!!!!!!!!!!!!!!!!!!! SIM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
typedef struct branch_ BRANCH;
typedef struct bus_dsp_ BUS_DSP;
typedef struct branch_dsp_ BRANCH_DSP;

typedef struct GeoLines_ GeoLines;
struct GeoLines_	
{
	float x;
	float y;
	int	  numofseg;
};

/*typedef struct GeoData_ {
	float *xcoords;
	float *ycoords;
	
	float xcoordmax;
	float ycoordmax;
	
	float ycoordmin;
	float xcoordmin;
	
	int   datatype;
	int   numofpoints;
}GeoData;*/

struct branch_ {
    GLint tap_bus;
    GLint z_bus;
    GLint load_flow_area;
    GLint loss_zone;
    GLint circuit;
    GLint type;
    GLfloat resistance;
    GLfloat reactance;
    GLfloat charging;
    GLint MVA_1;
    GLint MVA_2;
    GLint MVA_3;
    GLint control_bus;
    GLint side;
    GLfloat turns_ratio;
    GLfloat angle;
    GLfloat min_tap;
    GLfloat max_tap;
    GLfloat step;
    GLfloat min_voltage;
    GLfloat max_voltage;
    GLfloat color[3];
};

//Dispatch Information for the Buses
struct bus_dsp_ {
    GLfloat pot_L[24]; // Potência de Carga
    GLfloat pot_G[24]; // Potência de Geração
};

//Dispatch Information for the Branches
struct branch_dsp_ {
    GLfloat max_pot;
    GLfloat pot[24];
    GLfloat pot_perc[24];
};

/////////////////////!!!!!!!!!!!!!!!!!!!!!! SIM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#define MAXLINELENGTH 153
#define TIMEFRAME 24
#define BUFSIZE 1024
#define ABS(x) ((x)<0?-(x):(x))
#define MAX(x,y) ((x)<(y)?(y):(x))

typedef struct title_ TITLE;
typedef struct bus_ BUS;
typedef struct Gbus_ GBUS;
typedef struct Gbranch_ GBRANCH;

struct title_ {
    char date[8];
    char originator[19];
    GLfloat MVA;
    GLint year;
    char season[1];
    char id[30];
};

struct bus_ {
    GLint number;
    char name[12];
    GLint load_flow_area;
    GLint loss_zone;
    GLint type;
    GLfloat voltage;
    GLfloat angle;
    GLfloat load_MW;
    GLfloat load_MVAR;
    GLfloat gen_MW;
    GLfloat gen_MVAR;
    GLfloat base_KV;
    GLfloat des_volts;
    GLfloat max_voltage;
    GLfloat min_voltage;
    GLfloat conductance;
    GLfloat susceptance;
    GLint remote;
    GLint left;
    GLint right;
    GLint branches;
    GLint buses[BUS_NUM];
};

//Graphical Information for the Buses
struct Gbus_ {
    GLfloat xi; //X-coordinate of the Bus initial point
    GLfloat yi; //Y-coordinate of the Bus initial point
    GLfloat xf; //X-coordinate of the Bus final point
    GLfloat yf; //Y-coordinate of the Bus final point
    GLint Gen_dir; //Direction of the Generator
    GLfloat Gen_x; //X-coordinate of the Generator initial point
    GLfloat Gen_y; //Y-coordinate of the Generator initial point
    GLint Load_dir; //Direction of the Load
    GLfloat Load_x; //X-coordinate of the Load initial point
    GLfloat Load_y; //Y-coordinate of the Load initial point
    GLint Suscep_dir; //Direction of the Susceptance
    GLfloat Suscep_x; //X-coordinate of the Susceptance initial point
    GLfloat Suscep_y; //Y-coordinate of the Susceptance initial point
};

//Graphical Information for the Branches
struct Gbranch_ {
    GLint no_points;
    GLfloat pts_coords[20];
    GLfloat center[2];
};


/////
// Colors are global
/////
const GLfloat bkg1[4] = {1.0, 1.0, 1.0, 1.0}; /* Background Color 1 */
const GLfloat bkg2[4] = {0.0, 0.0, 0.0, 0.0}; /* Background Color 2 */
const GLfloat branch_color[3] = {0.0, 0.0, 1.0}; /* Branch Color */
const GLfloat problem_branch_color[3] = {1.0, 0.0, 0.0}; /* Problem Branch Color */
const GLfloat arrow_color[3] = {0.0, 1.0, 0.0}; /* Bus Color */
const GLfloat axes_color[3] = {0.8, 0.8, 0.8}; /* Axes Color */
const GLfloat graphic_color[3] = {0.0, 0.0, 1.0}; /* Graphic Color */
const GLfloat title_color[3] = {0.0, 0.5, 0.7}; /* Title Color */

/////
// DB Options (char *)
/////
#define DB_SERVER "localhost"
#define DB_SERVER_PORT ""
#define DB_USER "rajiv"

#endif //VPDEFS_H
